#include "botdll.h"

#include "datatypes.h"
#include "algorithms.h"
#include "host.h"
#include "clientprot.h"
#include "botdb.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>


void DLLImports::clearImports()
{
	if (DLL_TALK)
	{
		talk(makeTerm());
		DLL_TALK = NULL;
	}
}

void __stdcall listen(BotEvent &event)
{
	Host *h = (Host*)event.handle;

	switch (event.code)
	{
	case EVENT_Echo:
		{
			char *msg = (char*)event.p[0];

			h->logEvent("Ext: %s", msg);
		}
		break;
	case EVENT_Say:
		{
			int mode = *(int*)&event.p[0];
			int sound = *(int*)&event.p[1];
			int ident = *(int*)&event.p[2];
			char *msg = (char*)event.p[3];

			h->tryChat(mode, sound, ident, msg);
		}
		break;
	case EVENT_Ship:
		{
			int ship = *(int*)&event.p[0];

			h->postRR(generateChangeShip((Ship_Types)ship));
		}
		break;
	case EVENT_GrabFlag:
		{
			int flag = *(int*)&event.p[0];

			h->postRR(generateFlagRequest(flag));
		}
		break;
	case EVENT_SendPosition:
		{
			h->sendPosition();
		}
		break;
	case EVENT_DropFlags:
		{
			h->postRR(generateFlagDrop());
		}
		break;
	case EVENT_ChangeArena:
		{
			String name = (char*)event.p[0];

			if (!invalidArena(name))
				h->changeArena(name);
		}
		break;
	case EVENT_SpawnBot:
		{
			if (h->botInfo.db->spawns.getConnections() >= h->botInfo.maxSpawns)
				break;

			BOT_INFO bi;
			bi.set(h->botInfo);
 
			String name		= (char*)event.p[0];
			String password = (char*)event.p[1];
			String staff	= (char*)event.p[2];
			String arena	= (char*)event.p[3];

			// Name
			if (invalidName(name))
				break;

			// Password
			if (password.IsEmpty())
				password = bi.password;

			// Staff
			if (staff.IsEmpty())
				staff = bi.staffpw;

			// Arena
			if (arena.IsEmpty())
				arena = bi.initialArena;

			if (invalidArena(arena))
				break;

			bi.setLogin(name, password, staff);
			bi.setArena(arena, bi.initialShip, bi.xres, bi.yres, bi.allowAudio);

			bi.db->spawns.connectHost(bi);
		}
		break;
	default:
		{
			String s;
			s += "Unknown message from DLL (";
			s += event.code;
			s += ")  Version mismatch?";
			h->logEvent("%s", s);
		}
		break;
	}
}

bool DLLImports::importLibrary(char *file)
{
	String s = file;

	// Avoid directory traversal exploits.
	trimString('/', s);
	trimString('\\', s);

	// Make sure that a .DLL extension is used.
	tolower(s.msg);
	if (s.right(4) != ".dll")
		s += ".dll";

	HMODULE DLLhMod = LoadLibrary(file);

	if (DLLhMod)
	{
		DLL_TALK = (CALL_TALK)GetProcAddress(DLLhMod, (LPCSTR)1);

		talk(makeInit(&listen, &h->playerlist, &h->flagList));
		if (h->inArena)
			talk(makeArenaEnter(h->botInfo.initialArena, h->Me));

		return true;
	}

	return false;
}

void DLLImports::talk(BotEvent &event)
{
	if (DLL_TALK)
	{
		event.handle = h;

		DLL_TALK(event);
	}
}

DLLImports::DLLImports(Host &host)
{
	DLL_TALK = NULL;

	h = &host;
}

DLLImports::DLLImports(DLLImports &imports)
{
	DLL_TALK = imports.DLL_TALK;

	h = imports.h;
}

DLLImports::~DLLImports()
{
	clearImports();
}
